<?php

	require_once "XML/Parser.php";

	/**
	 * Yet another XML parsing class 
	 *
	 * This is similar to the "func" mode of XML_Parser but it borrows
	 * some concepts from DSSSL.
	 * The tag handler method to call is not only determined by the tag
	 * name but also potentially by the name of its parent tags, and the
	 * most specific handler method (that is the one including the
	 * maximum number of matching parent tags in its name) wins.
	 * This way it is possible to have e.g. tagstart_name as a general
	 * handler for a <name> tag while tagstart_function_name handles the
	 * more special case of a <name> tag within a <function> tag.
	 * Character data within a tag is collected and passed to the end
	 * tag handler.
	 * Tag names and attributes are managed using stack arrays.
	 * Attributes are not only passed to both the start and end tag 
	 * handlers.
	 *
	 * @access public
	 */

	class PECL_Gen_XmlParser extends XML_Parser
  {
		/**
		 * We collect cData in here
		 *
		 * @access private
		 * @var    string
		 */
		var $data = "";

		/** 
		 * We maintain the current tag nesting structure here
		 *
		 * @access private
		 * @var    array
		 */
		var $tag_stack = array();

		/** 
		 * We keep track of tag attributes so that we can also provide them to the end tag handlers
		 *
		 * @access private
		 * @var    array
		 */
		var $attr_stack = array();

		/**
		 * There is no clean way to terminate parsing from within a handler ...
		 *
		 * @access private
		 * @var    bool
		 */
		var $error = false;

		/**
		 * We want a case sensitive parser (overriding XMP_Parser settings)
		 *
		 * @access private
		 * @var    bool
		 */
		var $folding = false;

		/**
		 * The constructor 
		 *
		 * @access public
		 */
	 	function __construct()
		{ 
      parent::XML_Parser();
    }

		/**
		 * Overload parse() for error detection 
		 *
		 * @access public
		 * @return boolean true on success
		 */
		function parse() 
		{
			$err = parent::parse();
			if ($this->error) {
				return $this->error;
			}
			return $err;
		}

		/**
		 * Try to find best matching tag handler for current tag nesting
		 * 
		 * @access private
		 * @param  string  handler method prefix
		 * @return string  hndler method name or false if no handler found
		 */
		function findHandler($prefix)
		{
			for ($tags = $this->tag_stack; count($tags); array_shift($tags)) {
				$method = "{$prefix}_".join("_", $tags);
				if (method_exists($this, $method)) {
					return $method;
				}
			}
			return false;
		}


		/**
		 * Try to find a tagstart handler for this tag and call it
		 *
		 * @access private
		 * @param  resource internal parser handle
		 * @param  string   tag name
		 * @param  array    tag attributes
		 */
		function startHandler($XmlParser, $tag, $attr)
		{
			if ($this->error) return;

			$this->data = "";
			array_push($this->tag_stack, $tag);
			array_push($this->attr_stack, $attr);

			$method = $this->findHandler("tagstart");
			if ($method) {
				$err = $this->$method($attr);
				if (PEAR::isError($err)) {
					$this->error = $err;
					$this->error->addUserInfo("in XML input on line ".                                   
																		xml_get_current_line_number($XmlParser).
																		":".
																		xml_get_current_column_number($XmlParser));
				}
			}
		}

		/**
		 * Try to find a tagend handler for this tag and call it
		 *
		 * @access private
		 * @param  resource internal parser handle
		 * @param  string   tag name
		 */
		function endHandler($XmlParser, $tag)
		{
			if ($this->error) return;

			$method = $this->findHandler("tagend");
			array_pop($this->tag_stack);
			$attr = array_pop($this->attr_stack);
			if ($method) {
				$err = $this->$method($attr, $this->data);
				if (PEAR::isError($err)) {
					$this->error = $err;
					$this->error->addUserInfo("in XML input on line ".                                   
																		xml_get_current_line_number($XmlParser).
																		":".
																		xml_get_current_column_number($XmlParser));
				}
			}
			$this->data = "";
		}


		/**
		 * Just collect cData for later use in tag end handlers
		 *
		 * @access private
		 * @param  resource internal parser handle
		 * @param  string   cData to collect
		 */
		function cDataHandler($XmlParser, $data)
		{
			$this->data.= $data;
		}

  }
?>